import { defineComponent, openBlock, createBlock, resolveDynamicComponent, withCtx, createElementVNode, renderSlot, createElementBlock, Fragment, renderList, mergeProps, toHandlers, toDisplayString, normalizeClass, createVNode, Transition, withDirectives, vShow } from "vue";
function isTouchSupport() {
  return "ontouchstart" in window || !!navigator.maxTouchPoints;
}
function touchEvent() {
  return isTouchSupport() ? "touchend" : "click";
}
function pointerEvents() {
  return window.PointerEvent ? {
    end: "pointerup",
    enter: "pointerenter",
    leave: "pointerleave"
  } : {
    end: "touchend",
    enter: "mouseenter",
    leave: "mouseleave"
  };
}
function isOutsideClick(element, event) {
  return element === event.target || !element.contains(event.target);
}
var _export_sfc = (sfc, props) => {
  const target = sfc.__vccOpts || sfc;
  for (const [key, val] of props) {
    target[key] = val;
  }
  return target;
};
const _sfc_main$1 = defineComponent({
  name: "VsmMenu",
  props: {
    menu: {
      type: Array,
      required: true
    },
    element: {
      type: String,
      default: "header",
      validator: (val) => !!val
    },
    screenOffset: {
      type: [Number, String],
      default: 10,
      validator: (val) => 0 <= +val
    },
    dropdownOffset: {
      type: [Number, String],
      default: 0,
      validator: (val) => 0 <= +val
    },
    handler: {
      type: String,
      default: "hover",
      validator: (val) => ["hover", "click"].includes(val)
    },
    transitionTimeout: {
      type: [Number, String],
      default: 250
    },
    align: {
      type: String,
      default: "center",
      validator: (val) => ["left", "center", "right"].includes(val)
    }
  },
  emits: ["open-dropdown", "close-dropdown"],
  data() {
    return {
      activeDropdown: void 0,
      activeContainerItem: void 0,
      elementsWithDropdown: [],
      dropdownContainerItems: [],
      pointerEvent: {},
      isDragging: false,
      closeDropdownTimeout: void 0,
      enableTransitionTimeout: void 0,
      disableTransitionTimeout: void 0,
      lastWindowWidth: void 0
    };
  },
  computed: {
    itemsWithDropdown() {
      return this.menu.filter((item) => item.dropdown);
    }
  },
  watch: {
    async element() {
      await this.$nextTick();
      this.updateDataElements();
      this.registerDropdownElementsEvents();
    },
    handler() {
      this.registerDropdownElementsEvents(true);
      this.registerDropdownContainerEvents(true);
    },
    menu: {
      async handler() {
        await this.$nextTick();
        this.updateDataElements();
        this.registerDropdownElementsEvents();
      },
      deep: true
    }
  },
  mounted() {
    this.identifyPointerEvents();
    this.registerGlobalListeners();
    this.updateDataElements();
    this.registerDropdownElementsEvents();
    this.registerDropdownContainerEvents();
  },
  beforeUnmount() {
    this.removeGlobalListeners();
  },
  methods: {
    toggleDropdown(element) {
      if (this.activeDropdown === element) {
        this.closeDropdown();
      } else {
        this.openDropdown(element);
      }
    },
    openDropdown(element) {
      if (this.activeDropdown === element) {
        return;
      }
      this.$emit("open-dropdown", element);
      this.$el.classList.add("vsm-overlay-active", "vsm-dropdown-active");
      this.activeDropdown = element;
      this.activeDropdown.setAttribute("aria-expanded", "true");
      this.elementsWithDropdown.forEach((el) => el.classList.remove("vsm-active"));
      element.classList.add("vsm-active");
      const activeDataDropdown = element.getAttribute("data-dropdown");
      let direction = "vsm-left";
      this.dropdownContainerItems.forEach((item) => {
        item.el.classList.remove("vsm-active", "vsm-left", "vsm-right");
        if (item.name === activeDataDropdown) {
          item.el.setAttribute("aria-hidden", "false");
          item.el.classList.add("vsm-active");
          direction = "vsm-right";
          this.activeContainerItem = item;
        } else {
          item.el.setAttribute("aria-hidden", "true");
          item.el.classList.add(direction);
        }
      });
      this.resizeDropdown();
    },
    closeDropdown() {
      if (!this.activeDropdown || !this.activeContainerItem) {
        return;
      }
      this.$emit("close-dropdown", this.activeDropdown);
      this.elementsWithDropdown.forEach((el) => el.classList.remove("vsm-active"));
      this.activeContainerItem.el.setAttribute("aria-hidden", "true");
      this.clearEnableTransitionTimeout();
      this.startDisableTransitionTimeout();
      this.$el.classList.remove("vsm-overlay-active", "vsm-dropdown-active");
      this.activeDropdown.setAttribute("aria-expanded", "false");
      this.activeContainerItem = void 0;
      this.activeDropdown = void 0;
    },
    resizeDropdown() {
      if (!this.activeDropdown || !this.activeContainerItem) {
        return;
      }
      const bodyWidth = document.documentElement.offsetWidth;
      const rootRect = this.$el.getBoundingClientRect();
      const rect = this.activeDropdown.getBoundingClientRect();
      const { offsetHeight } = this.activeContainerItem.content;
      let { offsetWidth } = this.activeContainerItem.content;
      const startPosition = rect.left - rootRect.left;
      let position = null;
      switch (this.activeContainerItem.align || this.align) {
        case "left":
          position = startPosition;
          break;
        case "right":
          position = startPosition - offsetWidth + rect.width;
          break;
        default:
          position = startPosition - offsetWidth / 2 + rect.width / 2;
      }
      if (position + rootRect.left < +this.screenOffset) {
        position = +this.screenOffset - rootRect.left;
      }
      const rightOffset = position + rootRect.left + offsetWidth;
      if (rightOffset > bodyWidth - +this.screenOffset) {
        position -= rightOffset - bodyWidth + +this.screenOffset;
        if (position < +this.screenOffset - rootRect.left) {
          position = +this.screenOffset - rootRect.left;
          offsetWidth = bodyWidth - +this.screenOffset * 2;
        }
      }
      position = Math.round(position);
      const dropdownOffset = +this.dropdownOffset + this.activeDropdown.offsetTop;
      this.clearDisableTransitionTimeout();
      this.startEnableTransitionTimeout();
      const dropdownContainer = this.$refs.dropdownContainer;
      dropdownContainer.style.transform = `translate(${position}px, ${dropdownOffset}px)`;
      dropdownContainer.style.width = `${offsetWidth}px`;
      dropdownContainer.style.height = `${offsetHeight}px`;
      this.$refs.arrow.style.transform = `translate(${Math.round(
        startPosition + rect.width / 2
      )}px, ${dropdownOffset}px) rotate(45deg)`;
      this.$refs.background.style.transform = `translate(${position}px, ${dropdownOffset}px)`;
      this.$refs.background.style.width = `${offsetWidth}px`;
      this.$refs.background.style.height = `${offsetHeight}px`;
      this.$refs.backgroundAlt.style.transform = `translateY(${this.activeContainerItem.content.firstElementChild.offsetHeight}px)`;
    },
    startCloseDropdownTimeout() {
      this.closeDropdownTimeout = setTimeout(() => this.closeDropdown(), 50);
    },
    clearCloseDropdownTimeout() {
      clearTimeout(this.closeDropdownTimeout);
    },
    startEnableTransitionTimeout() {
      this.enableTransitionTimeout = setTimeout(() => this.$el.classList.remove("vsm-no-transition"), 50);
    },
    clearEnableTransitionTimeout() {
      clearTimeout(this.enableTransitionTimeout);
    },
    startDisableTransitionTimeout() {
      this.disableTransitionTimeout = setTimeout(
        () => this.$el.classList.add("vsm-no-transition"),
        +this.transitionTimeout
      );
    },
    clearDisableTransitionTimeout() {
      clearTimeout(this.disableTransitionTimeout);
    },
    registerDropdownElementsEvents(force = false) {
      this.elementsWithDropdown.forEach((el) => {
        if (el._vsmMenu && !force) {
          return;
        }
        if (el._vsmMenuHandlers) {
          Object.entries(el._vsmMenuHandlers).forEach(([eventName, fn]) => {
            el.removeEventListener(eventName, fn);
          });
        }
        if ("hover" === this.handler) {
          el._vsmMenuHandlers = {
            focusin: () => {
              this.clearCloseDropdownTimeout();
              this.openDropdown(el);
            },
            [this.pointerEvent.enter]: (evt) => {
              if ("touch" !== evt.pointerType) {
                this.clearCloseDropdownTimeout();
                this.openDropdown(el);
              }
            },
            [this.pointerEvent.leave]: (evt) => {
              if ("touch" !== evt.pointerType) {
                this.startCloseDropdownTimeout();
              }
            }
          };
        } else {
          el._vsmMenuHandlers = {};
        }
        el._vsmMenuHandlers[this.pointerEvent.end] = (evt) => {
          evt.preventDefault();
          evt.stopPropagation();
          this.toggleDropdown(el);
        };
        Object.entries(el._vsmMenuHandlers).forEach(([eventName, fn]) => {
          el.addEventListener(eventName, fn);
        });
        el._vsmMenu = true;
      });
    },
    registerDropdownContainerEvents(force = false) {
      const el = this.$refs.dropdownContainer;
      if (el._vsmMenu && !force) {
        return;
      }
      if (el._vsmMenuHandlers) {
        Object.entries(el._vsmMenuHandlers).forEach(([eventName, fn]) => {
          el.removeEventListener(eventName, fn);
        });
      }
      if ("hover" === this.handler) {
        el._vsmMenuHandlers = {
          [this.pointerEvent.enter]: (evt) => {
            if ("touch" !== evt.pointerType) {
              this.clearCloseDropdownTimeout();
            }
          },
          [this.pointerEvent.leave]: (evt) => {
            if ("touch" !== evt.pointerType) {
              this.startCloseDropdownTimeout();
            }
          }
        };
      } else {
        el._vsmMenuHandlers = {};
      }
      el._vsmMenuHandlers[this.pointerEvent.end] = (evt) => {
        evt.stopPropagation();
      };
      Object.entries(el._vsmMenuHandlers).forEach(([eventName, fn]) => {
        el.addEventListener(eventName, fn);
      });
      el._vsmMenu = true;
    },
    registerGlobalListeners() {
      window.addEventListener("resize", this.windowResizeHandler);
      document.addEventListener("touchmove", this.documentTouchMoveHandler);
      document.addEventListener("touchstart", this.documentTouchStartHandler);
      document.body.addEventListener(this.pointerEvent.end, this.documentEventEndHandler);
    },
    removeGlobalListeners() {
      window.removeEventListener("resize", this.windowResizeHandler);
      document.removeEventListener("touchmove", this.documentTouchMoveHandler);
      document.removeEventListener("touchstart", this.documentTouchStartHandler);
      document.body.removeEventListener(this.pointerEvent.end, this.documentEventEndHandler);
    },
    windowResizeHandler() {
      if (this.lastWindowWidth === window.innerWidth) {
        return;
      }
      this.lastWindowWidth = window.innerWidth;
      if (this.activeDropdown) {
        this.$el.classList.add("vsm-no-transition");
        this.resizeDropdown();
        return;
      }
      if (!this.$refs.background.getAttribute("style")) {
        return;
      }
      this.$el.classList.add("vsm-no-transition", "vsm-no-transition_dropdown");
      setTimeout(() => this.$el.classList.remove("vsm-no-transition_dropdown"));
      this.closeDropdown();
      this.clearAllStyles();
    },
    documentTouchMoveHandler() {
      this.isDragging = true;
    },
    documentTouchStartHandler() {
      this.isDragging = false;
    },
    documentEventEndHandler() {
      if (!this.isDragging) {
        this.closeDropdown();
      }
    },
    identifyPointerEvents() {
      this.pointerEvent = pointerEvents();
    },
    updateDataElements() {
      this.elementsWithDropdown = Array.from(this.$refs.linkContainer.children).filter(
        (el) => el.classList.contains("vsm-has-dropdown")
      );
      this.dropdownContainerItems = Array.from(this.$refs.dropdownContainer.children).map((el) => ({
        el,
        name: el.getAttribute("data-dropdown"),
        align: el.getAttribute("data-align"),
        content: el.firstElementChild
      }));
    },
    clearAllStyles() {
      this.$refs.dropdownContainer.removeAttribute("style");
      this.$refs.arrow.removeAttribute("style");
      this.$refs.background.removeAttribute("style");
      this.$refs.backgroundAlt.removeAttribute("style");
    }
  }
});
const _hoisted_1$1 = { class: "vsm-nav" };
const _hoisted_2$1 = {
  ref: "root",
  class: "vsm-root"
};
const _hoisted_3$1 = {
  ref: "linkContainer",
  class: "vsm-link-container vsm-mob-hide"
};
const _hoisted_4 = { class: "vsm-dropdown vsm-mob-hide" };
const _hoisted_5 = {
  ref: "background",
  class: "vsm-background"
};
const _hoisted_6 = {
  ref: "backgroundAlt",
  class: "vsm-background-alt"
};
const _hoisted_7 = {
  ref: "arrow",
  class: "vsm-arrow"
};
const _hoisted_8 = {
  ref: "dropdownContainer",
  class: "vsm-dropdown-container"
};
const _hoisted_9 = ["data-dropdown", "data-align"];
const _hoisted_10 = { class: "vsm-dropdown-content" };
function _sfc_render$1(_ctx, _cache, $props, $setup, $data, $options) {
  return openBlock(), createBlock(resolveDynamicComponent(_ctx.element), { class: "vsm-menu vsm-no-transition" }, {
    default: withCtx(() => [
      createElementVNode("nav", _hoisted_1$1, [
        createElementVNode("ul", _hoisted_2$1, [
          renderSlot(_ctx.$slots, "before-nav"),
          createElementVNode("li", _hoisted_3$1, [
            (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.menu, (item, index2) => {
              var _a;
              return openBlock(), createBlock(resolveDynamicComponent(item.element || (item.dropdown ? "button" : "a")), mergeProps({
                key: index2,
                class: [
                  "vsm-link",
                  (_a = item.attributes) == null ? void 0 : _a.class,
                  {
                    "vsm-has-dropdown": item.dropdown
                  }
                ],
                "data-dropdown": item.dropdown,
                "data-align": item.align,
                "aria-haspopup": item.dropdown && "true",
                "aria-expanded": item.dropdown && "false",
                tabindex: "0"
              }, item.attributes, toHandlers(item.listeners || {})), {
                default: withCtx(() => [
                  renderSlot(_ctx.$slots, "title", {
                    item,
                    index: index2
                  }, () => [
                    createElementVNode("span", null, toDisplayString(item.title), 1)
                  ])
                ]),
                _: 2
              }, 1040, ["class", "data-dropdown", "data-align", "aria-haspopup", "aria-expanded"]);
            }), 128))
          ], 512),
          renderSlot(_ctx.$slots, "after-nav")
        ], 512)
      ]),
      createElementVNode("div", _hoisted_4, [
        createElementVNode("div", _hoisted_5, [
          createElementVNode("div", _hoisted_6, null, 512)
        ], 512),
        createElementVNode("div", _hoisted_7, null, 512),
        createElementVNode("div", _hoisted_8, [
          (openBlock(true), createElementBlock(Fragment, null, renderList(_ctx.itemsWithDropdown, (item, index2) => {
            return openBlock(), createElementBlock("div", {
              key: index2,
              class: "vsm-dropdown-section",
              "data-dropdown": item.dropdown,
              "data-align": item.align,
              "aria-hidden": "false"
            }, [
              createElementVNode("div", _hoisted_10, [
                renderSlot(_ctx.$slots, "default", {
                  item,
                  index: index2
                })
              ])
            ], 8, _hoisted_9);
          }), 128))
        ], 512)
      ])
    ]),
    _: 3
  });
}
var VsmMenu = /* @__PURE__ */ _export_sfc(_sfc_main$1, [["render", _sfc_render$1]]);
const _sfc_main = defineComponent({
  name: "VsmMob",
  props: {
    value: {
      type: Boolean,
      default: false
    }
  },
  emits: ["input", "update:modelValue"],
  data() {
    return {
      active: this.value,
      event: "click"
    };
  },
  watch: {
    value(val) {
      if (this.active !== val) {
        this.active = val;
      }
    },
    active(val) {
      if (val) {
        this.registerEvent();
      } else {
        this.unregisterEvent();
      }
    }
  },
  mounted() {
    this.reloadEvent();
  },
  beforeUnmount() {
    this.unregisterEvent();
  },
  methods: {
    closeDropdown() {
      this.emitValue(false);
    },
    onClickHamburger() {
      this.emitValue(!this.active);
    },
    reloadEvent() {
      this.event = touchEvent();
    },
    registerEvent() {
      document.body.addEventListener(this.event, this.eventEndHandler);
    },
    unregisterEvent() {
      document.body.removeEventListener(this.event, this.eventEndHandler);
    },
    emitValue(toggle) {
      this.active = toggle;
      this.$emit("input", toggle);
      this.$emit("update:modelValue", toggle);
    },
    eventEndHandler(evt) {
      if (isOutsideClick(this.$el, evt)) {
        this.emitValue(false);
      }
    }
  }
});
const _hoisted_1 = /* @__PURE__ */ createElementVNode("div", { class: "vsm-mob__hamburger" }, [
  /* @__PURE__ */ createElementVNode("div", { class: "vsm-mob-line" }),
  /* @__PURE__ */ createElementVNode("div", { class: "vsm-mob-line" }),
  /* @__PURE__ */ createElementVNode("div", { class: "vsm-mob-line" })
], -1);
const _hoisted_2 = { class: "vsm-mob-content" };
const _hoisted_3 = { class: "vsm-mob-content__wrap" };
function _sfc_render(_ctx, _cache, $props, $setup, $data, $options) {
  return openBlock(), createElementBlock("li", {
    class: normalizeClass([
      "vsm-mob-container",
      "vsm-mob-show",
      {
        "vsm-open": _ctx.active
      }
    ])
  }, [
    createElementVNode("div", {
      class: "vsm-mob",
      onClick: _cache[0] || (_cache[0] = (...args) => _ctx.onClickHamburger && _ctx.onClickHamburger(...args))
    }, [
      renderSlot(_ctx.$slots, "hamburger", {}, () => [
        _hoisted_1
      ])
    ]),
    createElementVNode("div", _hoisted_2, [
      createVNode(Transition, { name: "vsm-mob-anim" }, {
        default: withCtx(() => [
          withDirectives(createElementVNode("div", _hoisted_3, [
            renderSlot(_ctx.$slots, "close", {}, () => [
              createElementVNode("div", {
                class: "vsm-mob-close",
                onClick: _cache[1] || (_cache[1] = (...args) => _ctx.onClickHamburger && _ctx.onClickHamburger(...args))
              })
            ]),
            renderSlot(_ctx.$slots, "default")
          ], 512), [
            [vShow, _ctx.active]
          ])
        ]),
        _: 3
      })
    ])
  ], 2);
}
var VsmMob = /* @__PURE__ */ _export_sfc(_sfc_main, [["render", _sfc_render]]);
var index$1 = "";
function install(Vue) {
  Vue.component(VsmMenu.name, VsmMenu);
  Vue.component(VsmMob.name, VsmMob);
}
var index = { install };
export { VsmMenu, VsmMob, index as default };
